@@ -273,13 +273,18 @@ class QueryParameters(object):
 
         :param mappings: Key-value pairs where the key is the client-side
                          name we'll accept here and the value is the name
-                         the server expects, e.g, changes_since=changes-since
+                         the server expects, e.g, changes_since=changes-since.
+                         Additionally, a value can be a dict with optional keys
+                         name - server-side name,
+                         type - callable to convert from client to server
+                         representation.
 
         By default, both limit and marker are included in the initial mapping
         as they're the most common query parameters used for listing resources.
         """
         self._mapping = {"limit": "limit", "marker": "marker"}
-        self._mapping.update(dict({name: name for name in names}, **mappings))
+        self._mapping.update({name: name for name in names})
+        self._mapping.update(mappings)
 
     def _validate(self, query, base_path=None):
         """Check that supplied query keys match known query mappings
@@ -290,7 +295,9 @@ class QueryParameters(object):
                           the resource.
         """
         expected_params = list(self._mapping.keys())
-        expected_params += self._mapping.values()
+        expected_params.extend(
+            value['name'] if isinstance(value, dict) else value
+            for value in self._mapping.values())
 
         if base_path:
             expected_params += utils.get_string_format_keys(base_path)
@@ -312,11 +319,25 @@ class QueryParameters(object):
                            server side name.
         """
         result = {}
-        for key, value in self._mapping.items():
-            if key in query:
-                result[value] = query[key]
-            elif value in query:
-                result[value] = query[value]
+        for client_side, server_side in self._mapping.items():
+            if isinstance(server_side, dict):
+                name = server_side['name']
+                type_ = server_side.get('type')
+            else:
+                name = server_side
+                type_ = None
+
+            if client_side in query:
+                value = query[client_side]
+            elif name in query:
+                value = query[name]
+            else:
+                continue
+
+            if type_ is not None:
+                result[name] = type_(value)
+            else:
+                result[name] = value
         return result
 
 
@@ -394,6 +415,7 @@ class Resource(dict):
     _uri = None
     _computed = None
     _original_body = None
+    _delete_response_class = None
 
     def __init__(self, _synchronized=False, connection=None, **attrs):
         """The base resource
@@ -579,8 +601,7 @@ class Resource(dict):
         # TODO(mordred) We should make a Location Resource and add it here
         # instead of just the dict.
         if self._connection:
-            computed['location'] = munch.unmunchify(
-                self._connection._openstackcloud.current_location)
+            computed['location'] = self._connection.current_location
 
         return body, header, uri, computed
 
@@ -745,11 +766,7 @@ class Resource(dict):
         :param dict kwargs: Each of the named arguments will be set as
                             attributes on the resulting Resource object.
         """
-        res = cls(_synchronized=True, **kwargs)
-        # TODO(shade) Done as a second call rather than a constructor param
-        # because otherwise the mocking in the tests goes nuts.
-        res._connection = connection
-        return res
+        return cls(_synchronized=True, connection=connection, **kwargs)
 
     @classmethod
     def _from_munch(cls, obj, synchronized=True, connection=None):
@@ -765,7 +782,7 @@ class Resource(dict):
         return cls(_synchronized=synchronized, connection=connection, **obj)
 
     def to_dict(self, body=True, headers=True, computed=True,
-                ignore_none=False, original_names=False):
+                ignore_none=False, original_names=False, _to_munch=False):
         """Return a dictionary of this resource's contents
 
         :param bool body: Include the :class:`~openstack.resource.Body`
@@ -779,11 +796,16 @@ class Resource(dict):
                                  attributes that the server hasn't returned.
         :param bool original_names: When True, use attribute names as they
                                     were received from the server.
+        :param bool _to_munch: For internal use only. Converts to `munch.Munch`
+                               instead of dict.
 
         :return: A dictionary of key/value pairs where keys are named
                  as they exist as attributes of this class.
         """
-        mapping = {}
+        if _to_munch:
+            mapping = munch.Munch()
+        else:
+            mapping = {}
 
         components = []
         if body:
@@ -819,12 +841,17 @@ class Resource(dict):
                         if ignore_none and value is None:
                             continue
                         if isinstance(value, Resource):
-                            mapping[key] = value.to_dict()
+                            mapping[key] = value.to_dict(_to_munch=_to_munch)
+                        elif isinstance(value, dict) and _to_munch:
+                            mapping[key] = munch.Munch(value)
                         elif value and isinstance(value, list):
                             converted = []
                             for raw in value:
                                 if isinstance(raw, Resource):
-                                    converted.append(raw.to_dict())
+                                    converted.append(
+                                        raw.to_dict(_to_munch=_to_munch))
+                                elif isinstance(raw, dict) and _to_munch:
+                                    converted.append(munch.Munch(raw))
                                 else:
                                     converted.append(raw)
                             mapping[key] = converted
@@ -837,10 +864,11 @@ class Resource(dict):
     # Make the munch copy method use to_dict
     copy = to_dict
 
-    def _to_munch(self):
+    def _to_munch(self, original_names=True):
         """Convert this resource into a Munch compatible with shade."""
-        return munch.Munch(self.to_dict(body=True, headers=False,
-                                        original_names=True))
+        return self.to_dict(
+            body=True, headers=False,
+            original_names=original_names, _to_munch=True)
 
     def _prepare_request_body(self, patch, prepend_key):
         if patch:
@@ -853,7 +881,7 @@ class Resource(dict):
         return body
 
     def _prepare_request(self, requires_id=None, prepend_key=False,
-                         patch=False):
+                         patch=False, base_path=None):
         """Prepare a request to be sent to the server
 
         Create operations don't require an ID, but all others do,
@@ -881,7 +909,9 @@ class Resource(dict):
             else:
                 headers[k] = str(v)
 
-        uri = self.base_path % self._uri.attributes
+        if base_path is None:
+            base_path = self.base_path
+        uri = base_path % self._uri.attributes
         if requires_id:
             if self.id is None:
                 raise exceptions.InvalidRequest(
@@ -1016,7 +1046,7 @@ class Resource(dict):
 
         return actual
 
-    def create(self, session, prepend_key=True):
+    def create(self, session, prepend_key=True, base_path=None):
         """Create a remote resource based on this instance.
 
         :param session: The session to use for making this request.
@@ -1024,7 +1054,9 @@ class Resource(dict):
         :param prepend_key: A boolean indicating whether the resource_key
                             should be prepended in a resource creation
                             request. Default to True.
-
+        :param str base_path: Base part of the URI for creating resources, if
+                              different from
+                              :data:`~openstack.resource.Resource.base_path`.
         :return: This :class:`Resource` instance.
         :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                  :data:`Resource.allow_create` is not set to ``True``.
@@ -1036,13 +1068,15 @@ class Resource(dict):
         microversion = self._get_microversion_for(session, 'create')
         if self.create_method == 'PUT':
             request = self._prepare_request(requires_id=True,
-                                            prepend_key=prepend_key)
+                                            prepend_key=prepend_key,
+                                            base_path=base_path)
             response = session.put(request.url,
                                    json=request.body, headers=request.headers,
                                    microversion=microversion)
         elif self.create_method == 'POST':
             request = self._prepare_request(requires_id=False,
-                                            prepend_key=prepend_key)
+                                            prepend_key=prepend_key,
+                                            base_path=base_path)
             response = session.post(request.url,
                                     json=request.body, headers=request.headers,
                                     microversion=microversion)
@@ -1054,13 +1088,19 @@ class Resource(dict):
         self._translate_response(response)
         return self
 
-    def fetch(self, session, requires_id=True, error_message=None):
+    def fetch(self, session, requires_id=True,
+              base_path=None, error_message=None):
         """Get a remote resource based on this instance.
 
         :param session: The session to use for making this request.
         :type session: :class:`~keystoneauth1.adapter.Adapter`
         :param boolean requires_id: A boolean indicating whether resource ID
                                     should be part of the requested URI.
+        :param str base_path: Base part of the URI for fetching resources, if
+                              different from
+                              :data:`~openstack.resource.Resource.base_path`.
+        :param str error_message: An Error message to be returned if
+                                  requested object does not exist.
         :return: This :class:`Resource` instance.
         :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                  :data:`Resource.allow_fetch` is not set to ``True``.
@@ -1070,7 +1110,8 @@ class Resource(dict):
         if not self.allow_fetch:
             raise exceptions.MethodNotSupported(self, "fetch")
 
-        request = self._prepare_request(requires_id=requires_id)
+        request = self._prepare_request(requires_id=requires_id,
+                                        base_path=base_path)
         session = self._get_session(session)
         microversion = self._get_microversion_for(session, 'fetch')
         response = session.get(request.url, microversion=microversion)
@@ -1082,11 +1123,14 @@ class Resource(dict):
         self._translate_response(response, **kwargs)
         return self
 
-    def head(self, session):
+    def head(self, session, base_path=None):
         """Get headers from a remote resource based on this instance.
 
         :param session: The session to use for making this request.
         :type session: :class:`~keystoneauth1.adapter.Adapter`
+        :param str base_path: Base part of the URI for fetching resources, if
+                              different from
+                              :data:`~openstack.resource.Resource.base_path`.
 
         :return: This :class:`Resource` instance.
         :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
@@ -1097,7 +1141,7 @@ class Resource(dict):
         if not self.allow_head:
             raise exceptions.MethodNotSupported(self, "head")
 
-        request = self._prepare_request()
+        request = self._prepare_request(base_path=base_path)
 
         session = self._get_session(session)
         microversion = self._get_microversion_for(session, 'fetch')
@@ -1110,7 +1154,7 @@ class Resource(dict):
         return self
 
     def commit(self, session, prepend_key=True, has_body=True,
-               retry_on_conflict=None):
+               retry_on_conflict=None, base_path=None):
         """Commit the state of the instance to the remote resource.
 
         :param session: The session to use for making this request.
@@ -1121,6 +1165,9 @@ class Resource(dict):
         :param bool retry_on_conflict: Whether to enable retries on HTTP
                                        CONFLICT (409). Value of ``None`` leaves
                                        the `Adapter` defaults.
+        :param str base_path: Base part of the URI for modifying resources, if
+                              different from
+                              :data:`~openstack.resource.Resource.base_path`.
 
         :return: This :class:`Resource` instance.
         :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
@@ -1142,7 +1189,9 @@ class Resource(dict):
         if self.commit_jsonpatch:
             kwargs['patch'] = True
 
-        request = self._prepare_request(prepend_key=prepend_key, **kwargs)
+        request = self._prepare_request(prepend_key=prepend_key,
+                                        base_path=base_path,
+                                        **kwargs)
         session = self._get_session(session)
 
         kwargs = {}
@@ -1188,16 +1237,8 @@ class Resource(dict):
         :raises: :exc:`~openstack.exceptions.ResourceNotFound` if
                  the resource was not found.
         """
-        if not self.allow_delete:
-            raise exceptions.MethodNotSupported(self, "delete")
 
-        request = self._prepare_request()
-        session = self._get_session(session)
-        microversion = self._get_microversion_for(session, 'delete')
-
-        response = session.delete(request.url,
-                                  headers={"Accept": ""},
-                                  microversion=microversion)
+        response = self._raw_delete(session)
         kwargs = {}
         if error_message:
             kwargs['error_message'] = error_message
@@ -1205,8 +1246,21 @@ class Resource(dict):
         self._translate_response(response, has_body=False, **kwargs)
         return self
 
+    def _raw_delete(self, session):
+        if not self.allow_delete:
+            raise exceptions.MethodNotSupported(self, "delete")
+
+        request = self._prepare_request()
+        session = self._get_session(session)
+        microversion = self._get_microversion_for(session, 'delete')
+
+        return session.delete(
+            request.url,
+            headers={"Accept": ""},
+            microversion=microversion)
+
     @classmethod
-    def list(cls, session, paginated=False, base_path=None, **params):
+    def list(cls, session, paginated=True, base_path=None, **params):
         """This method is a generator which yields resource objects.
 
         This resource object list generator handles pagination and takes query
@@ -1286,11 +1340,10 @@ class Resource(dict):
                 # argument and is practically a reserved word.
                 raw_resource.pop("self", None)
 
-                value = cls.existing(microversion=microversion, **raw_resource)
-                # TODO(shade) Done as a second call rather than a constructor
-                # param because otherwise the mocking in the tests goes nuts.
-                if hasattr(session, '_sdk_connection'):
-                    value._connection = session._sdk_connection
+                value = cls.existing(
+                    microversion=microversion,
+                    connection=session._get_connection(),
+                    **raw_resource)
                 marker = value.id
                 yield value
                 total_yielded += 1
@@ -1311,12 +1364,15 @@ class Resource(dict):
 
             if not pagination_key and 'links' in data:
                 # api-wg guidelines are for a links dict in the main body
-                pagination_key == 'links'
+                pagination_key = 'links'
             if not pagination_key and cls.resources_key:
                 # Nova has a {key}_links dict in the main body
                 pagination_key = '{key}_links'.format(key=cls.resources_key)
             if pagination_key:
                 links = data.get(pagination_key, {})
+                # keystone might return a dict
+                if isinstance(links, dict):
+                    links = ({k: v} for k, v in six.iteritems(links))
                 for item in links:
                     if item.get('rel') == 'next' and 'href' in item:
                         next_link = item['href']
@@ -1395,13 +1451,13 @@ class Resource(dict):
         :raises: :class:`openstack.exceptions.ResourceNotFound` if nothing
                  is found and ignore_missing is ``False``.
         """
+        session = cls._get_session(session)
         # Try to short-circuit by looking directly for a matching ID.
         try:
-            match = cls.existing(id=name_or_id, **params)
-            # TODO(shade) Done as a second call rather than a constructor
-            # param because otherwise the mocking in the tests goes nuts.
-            if hasattr(session, '_sdk_connection'):
-                match._connection = session._sdk_connection
+            match = cls.existing(
+                id=name_or_id,
+                connection=session._get_connection(),
+                **params)
             return match.fetch(session)
         except exceptions.NotFoundException:
             pass
@@ -1418,6 +1474,119 @@ class Resource(dict):
             "No %s found for %s" % (cls.__name__, name_or_id))
 
 
+class TagMixin(object):
+
+    _tag_query_parameters = {
+        'tags': 'tags',
+        'any_tags': 'tags-any',
+        'not_tags': 'not-tags',
+        'not_any_tags': 'not-tags-any',
+    }
+
+    #: A list of associated tags
+    #: *Type: list of tag strings*
+    tags = Body('tags', type=list, default=[])
+
+    def fetch_tags(self, session):
+        """Lists tags set on the entity.
+
+        :param session: The session to use for making this request.
+        :return: The list with tags attached to the entity
+        """
+        url = utils.urljoin(self.base_path, self.id, 'tags')
+        session = self._get_session(session)
+        response = session.get(url)
+        exceptions.raise_from_response(response)
+        # NOTE(gtema): since this is a common method
+        # we can't rely on the resource_key, because tags are returned
+        # without resource_key. Do parse response here
+        json = response.json()
+        if 'tags' in json:
+            self._body.attributes.update({'tags': json['tags']})
+        return self
+
+    def set_tags(self, session, tags=[]):
+        """Sets/Replaces all tags on the resource.
+
+        :param session: The session to use for making this request.
+        :param list tags: List with tags to be set on the resource
+        """
+        url = utils.urljoin(self.base_path, self.id, 'tags')
+        session = self._get_session(session)
+        response = session.put(url, json={'tags': tags})
+        exceptions.raise_from_response(response)
+        self._body.attributes.update({'tags': tags})
+        return self
+
+    def remove_all_tags(self, session):
+        """Removes all tags on the entity.
+
+        :param session: The session to use for making this request.
+        """
+        url = utils.urljoin(self.base_path, self.id, 'tags')
+        session = self._get_session(session)
+        response = session.delete(url)
+        exceptions.raise_from_response(response)
+        self._body.attributes.update({'tags': []})
+        return self
+
+    def check_tag(self, session, tag):
+        """Checks if tag exists on the entity.
+
+        If the tag does not exist a 404 will be returned
+
+        :param session: The session to use for making this request.
+        :param tag: The tag as a string.
+        """
+        url = utils.urljoin(self.base_path, self.id, 'tags', tag)
+        session = self._get_session(session)
+        response = session.get(url)
+        exceptions.raise_from_response(response,
+                                       error_message='Tag does not exist')
+        return self
+
+    def add_tag(self, session, tag):
+        """Adds a single tag to the resource.
+
+        :param session: The session to use for making this request.
+        :param tag: The tag as a string.
+        """
+        url = utils.urljoin(self.base_path, self.id, 'tags', tag)
+        session = self._get_session(session)
+        response = session.put(url)
+        exceptions.raise_from_response(response)
+        # we do not want to update tags directly
+        tags = self.tags
+        tags.append(tag)
+        self._body.attributes.update({
+            'tags': tags
+        })
+        return self
+
+    def remove_tag(self, session, tag):
+        """Removes a single tag from the specified server.
+
+        :param session: The session to use for making this request.
+        :param tag: The tag as a string.
+        """
+        url = utils.urljoin(self.base_path, self.id, 'tags', tag)
+        session = self._get_session(session)
+        response = session.delete(url)
+        exceptions.raise_from_response(response)
+        # we do not want to update tags directly
+        tags = self.tags
+        try:
+            # NOTE(gtema): if tags were not fetched, but request suceeded
+            # it is ok. Just ensure tag does not exist locally
+            tags.remove(tag)
+        except ValueError:
+            pass  # do nothing!
+        self._body.attributes.update({
+            'tags': tags
+        })
+        return self
+
+
 def _normalize_status(status):
     if status is not None:
         status = status.lower()
