@@ -58,14 +58,14 @@ class TestImageProxy(test_proxy_base.TestProxyBase):
 
     @mock.patch("openstack.resource.Resource._translate_response")
     @mock.patch("openstack.proxy.Proxy._get")
-    @mock.patch("openstack.image.v2.image.Image.update")
-    def test_image_update(self, mock_update_image, mock_get_image,
+    @mock.patch("openstack.image.v2.image.Image.commit")
+    def test_image_update(self, mock_commit_image, mock_get_image,
                           mock_transpose):
         original_image = image.Image(**EXAMPLE)
         mock_get_image.return_value = original_image
         EXAMPLE['name'] = 'fake_name'
         updated_image = image.Image(**EXAMPLE)
-        mock_update_image.return_value = updated_image.to_dict()
+        mock_commit_image.return_value = updated_image.to_dict()
         result = self.proxy.update_image(original_image,
                                          **updated_image.to_dict())
         self.assertEqual('fake_name', result.get('name'))
@@ -74,7 +74,7 @@ class TestImageProxy(test_proxy_base.TestProxyBase):
         self.verify_get(self.proxy.get_image, image.Image)
 
     def test_images(self):
-        self.verify_list(self.proxy.images, image.Image, paginated=True)
+        self.verify_list(self.proxy.images, image.Image)
 
     def test_add_tag(self):
         self._verify("openstack.image.v2.image.Image.add_tag",
@@ -151,6 +151,6 @@ class TestImageProxy(test_proxy_base.TestProxyBase):
                                        'image_id': 'image_id'})
 
     def test_members(self):
-        self.verify_list(self.proxy.members, member.Member, paginated=False,
+        self.verify_list(self.proxy.members, member.Member,
                          method_args=('image_1',),
                          expected_kwargs={'image_id': 'image_1'})
