@@ -11,6 +11,7 @@
 # under the License.
 
 import uuid
+import mock
 
 from openstack.load_balancer.v2 import _proxy
 from openstack.load_balancer.v2 import health_monitor
@@ -20,11 +21,15 @@ from openstack.load_balancer.v2 import listener
 from openstack.load_balancer.v2 import load_balancer as lb
 from openstack.load_balancer.v2 import member
 from openstack.load_balancer.v2 import pool
+from openstack.load_balancer.v2 import quota
+from openstack import proxy as proxy_base
 from openstack.tests.unit import test_proxy_base
 
 
 class TestLoadBalancerProxy(test_proxy_base.TestProxyBase):
 
+    LB_ID = uuid.uuid4()
+    LISTENER_ID = uuid.uuid4()
     POOL_ID = uuid.uuid4()
     L7_POLICY_ID = uuid.uuid4()
 
@@ -34,20 +39,55 @@ class TestLoadBalancerProxy(test_proxy_base.TestProxyBase):
 
     def test_load_balancers(self):
         self.verify_list(self.proxy.load_balancers,
-                         lb.LoadBalancer,
-                         paginated=True)
+                         lb.LoadBalancer)
 
     def test_load_balancer_get(self):
         self.verify_get(self.proxy.get_load_balancer,
                         lb.LoadBalancer)
 
+    def test_load_balancer_stats_get(self):
+        self.verify_get(self.proxy.get_load_balancer_statistics,
+                        lb.LoadBalancerStats,
+                        value=[self.LB_ID],
+                        expected_args=[lb.LoadBalancerStats],
+                        expected_kwargs={'lb_id': self.LB_ID,
+                                         'requires_id': False})
+
     def test_load_balancer_create(self):
         self.verify_create(self.proxy.create_load_balancer,
                            lb.LoadBalancer)
 
-    def test_load_balancer_delete(self):
-        self.verify_delete(self.proxy.delete_load_balancer,
-                           lb.LoadBalancer, True)
+    @mock.patch.object(proxy_base.Proxy, '_get_resource')
+    def test_load_balancer_delete_non_cascade(self, mock_get_resource):
+        fake_load_balancer = mock.Mock()
+        fake_load_balancer.id = "load_balancer_id"
+        mock_get_resource.return_value = fake_load_balancer
+        self._verify2("openstack.proxy.Proxy._delete",
+                      self.proxy.delete_load_balancer,
+                      method_args=["resource_or_id", True,
+                                   False],
+                      expected_args=[lb.LoadBalancer,
+                                     fake_load_balancer],
+                      expected_kwargs={"ignore_missing": True})
+        self.assertFalse(fake_load_balancer.cascade)
+        mock_get_resource.assert_called_once_with(lb.LoadBalancer,
+                                                  "resource_or_id")
+
+    @mock.patch.object(proxy_base.Proxy, '_get_resource')
+    def test_load_balancer_delete_cascade(self, mock_get_resource):
+        fake_load_balancer = mock.Mock()
+        fake_load_balancer.id = "load_balancer_id"
+        mock_get_resource.return_value = fake_load_balancer
+        self._verify2("openstack.proxy.Proxy._delete",
+                      self.proxy.delete_load_balancer,
+                      method_args=["resource_or_id", True,
+                                   True],
+                      expected_args=[lb.LoadBalancer,
+                                     fake_load_balancer],
+                      expected_kwargs={"ignore_missing": True})
+        self.assertTrue(fake_load_balancer.cascade)
+        mock_get_resource.assert_called_once_with(lb.LoadBalancer,
+                                                  "resource_or_id")
 
     def test_load_balancer_find(self):
         self.verify_find(self.proxy.find_load_balancer,
@@ -59,13 +99,20 @@ class TestLoadBalancerProxy(test_proxy_base.TestProxyBase):
 
     def test_listeners(self):
         self.verify_list(self.proxy.listeners,
-                         listener.Listener,
-                         paginated=True)
+                         listener.Listener)
 
     def test_listener_get(self):
         self.verify_get(self.proxy.get_listener,
                         listener.Listener)
 
+    def test_listener_stats_get(self):
+        self.verify_get(self.proxy.get_listener_statistics,
+                        listener.ListenerStats,
+                        value=[self.LISTENER_ID],
+                        expected_args=[listener.ListenerStats],
+                        expected_kwargs={'listener_id': self.LISTENER_ID,
+                                         'requires_id': False})
+
     def test_listener_create(self):
         self.verify_create(self.proxy.create_listener,
                            listener.Listener)
@@ -84,8 +131,7 @@ class TestLoadBalancerProxy(test_proxy_base.TestProxyBase):
 
     def test_pools(self):
         self.verify_list(self.proxy.pools,
-                         pool.Pool,
-                         paginated=True)
+                         pool.Pool)
 
     def test_pool_get(self):
         self.verify_get(self.proxy.get_pool,
@@ -110,7 +156,6 @@ class TestLoadBalancerProxy(test_proxy_base.TestProxyBase):
     def test_members(self):
         self.verify_list(self.proxy.members,
                          member.Member,
-                         paginated=True,
                          method_kwargs={'pool': self.POOL_ID},
                          expected_kwargs={'pool_id': self.POOL_ID})
 
@@ -150,8 +195,7 @@ class TestLoadBalancerProxy(test_proxy_base.TestProxyBase):
 
     def test_health_monitors(self):
         self.verify_list(self.proxy.health_monitors,
-                         health_monitor.HealthMonitor,
-                         paginated=True)
+                         health_monitor.HealthMonitor)
 
     def test_health_monitor_get(self):
         self.verify_get(self.proxy.get_health_monitor,
@@ -175,8 +219,7 @@ class TestLoadBalancerProxy(test_proxy_base.TestProxyBase):
 
     def test_l7_policies(self):
         self.verify_list(self.proxy.l7_policies,
-                         l7_policy.L7Policy,
-                         paginated=True)
+                         l7_policy.L7Policy)
 
     def test_l7_policy_get(self):
         self.verify_get(self.proxy.get_l7_policy,
@@ -201,7 +244,6 @@ class TestLoadBalancerProxy(test_proxy_base.TestProxyBase):
     def test_l7_rules(self):
         self.verify_list(self.proxy.l7_rules,
                          l7_rule.L7Rule,
-                         paginated=True,
                          method_kwargs={'l7_policy': self.L7_POLICY_ID},
                          expected_kwargs={'l7policy_id': self.L7_POLICY_ID})
 
@@ -238,3 +280,24 @@ class TestLoadBalancerProxy(test_proxy_base.TestProxyBase):
                       method_args=["RULE", self.L7_POLICY_ID],
                       expected_args=[l7_rule.L7Rule, "RULE"],
                       expected_kwargs={"l7policy_id": self.L7_POLICY_ID})
+
+    def test_quotas(self):
+        self.verify_list(self.proxy.quotas, quota.Quota)
+
+    def test_quota_get(self):
+        self.verify_get(self.proxy.get_quota, quota.Quota)
+
+    def test_quota_update(self):
+        self.verify_update(self.proxy.update_quota, quota.Quota)
+
+    def test_quota_default_get(self):
+        self._verify2("openstack.proxy.Proxy._get",
+                      self.proxy.get_quota_default,
+                      expected_args=[quota.QuotaDefault],
+                      expected_kwargs={'requires_id': False})
+
+    def test_quota_delete(self):
+        self.verify_delete(self.proxy.delete_quota, quota.Quota, False)
+
+    def test_quota_delete_ignore(self):
+        self.verify_delete(self.proxy.delete_quota, quota.Quota, True)
